# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class TaskExceptionGroupAggregate(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'exception_type': 'str',
        'ray_session_name': 'str',
        'metrics': 'TaskGroupedAggregateMetrics'
    }

    attribute_map = {
        'exception_type': 'exception_type',
        'ray_session_name': 'ray_session_name',
        'metrics': 'metrics'
    }

    def __init__(self, exception_type=None, ray_session_name=None, metrics=None, local_vars_configuration=None):  # noqa: E501
        """TaskExceptionGroupAggregate - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._exception_type = None
        self._ray_session_name = None
        self._metrics = None
        self.discriminator = None

        self.exception_type = exception_type
        self.ray_session_name = ray_session_name
        self.metrics = metrics

    @property
    def exception_type(self):
        """Gets the exception_type of this TaskExceptionGroupAggregate.  # noqa: E501


        :return: The exception_type of this TaskExceptionGroupAggregate.  # noqa: E501
        :rtype: str
        """
        return self._exception_type

    @exception_type.setter
    def exception_type(self, exception_type):
        """Sets the exception_type of this TaskExceptionGroupAggregate.


        :param exception_type: The exception_type of this TaskExceptionGroupAggregate.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and exception_type is None:  # noqa: E501
            raise ValueError("Invalid value for `exception_type`, must not be `None`")  # noqa: E501

        self._exception_type = exception_type

    @property
    def ray_session_name(self):
        """Gets the ray_session_name of this TaskExceptionGroupAggregate.  # noqa: E501


        :return: The ray_session_name of this TaskExceptionGroupAggregate.  # noqa: E501
        :rtype: str
        """
        return self._ray_session_name

    @ray_session_name.setter
    def ray_session_name(self, ray_session_name):
        """Sets the ray_session_name of this TaskExceptionGroupAggregate.


        :param ray_session_name: The ray_session_name of this TaskExceptionGroupAggregate.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and ray_session_name is None:  # noqa: E501
            raise ValueError("Invalid value for `ray_session_name`, must not be `None`")  # noqa: E501

        self._ray_session_name = ray_session_name

    @property
    def metrics(self):
        """Gets the metrics of this TaskExceptionGroupAggregate.  # noqa: E501


        :return: The metrics of this TaskExceptionGroupAggregate.  # noqa: E501
        :rtype: TaskGroupedAggregateMetrics
        """
        return self._metrics

    @metrics.setter
    def metrics(self, metrics):
        """Sets the metrics of this TaskExceptionGroupAggregate.


        :param metrics: The metrics of this TaskExceptionGroupAggregate.  # noqa: E501
        :type: TaskGroupedAggregateMetrics
        """
        if self.local_vars_configuration.client_side_validation and metrics is None:  # noqa: E501
            raise ValueError("Invalid value for `metrics`, must not be `None`")  # noqa: E501

        self._metrics = metrics

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TaskExceptionGroupAggregate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, TaskExceptionGroupAggregate):
            return True

        return self.to_dict() != other.to_dict()
